/****************************************************************************\
 *           Copyright (C) 2012 Texas Instruments Incorporated.             *
 *                           All Rights Reserved                            *
 *                                                                          *
 * GENERAL DISCLAIMER                                                       *
 * ------------------                                                       *
 * All software and related documentation is provided "AS IS" and without   *
 * warranty or support of any kind and Texas Instruments expressly disclaims*
 * all other warranties, express or implied, including, but not limited to, *
 * the implied warranties of merchantability and fitness for a particular   *
 * purpose.  Under no circumstances shall Texas Instruments be liable for   *
 * any incidental, special or consequential damages that result from the    *
 * use or inability to use the software or related documentation, even if   *
 * Texas Instruments has been advised of the liability.                     *
 ****************************************************************************
This example shows EMIF operations:
1. NOR FLASH test
2. NAND FLASH test
 ****************************************************************************
 * Created by :                                                             *
 *            Brighton Feng                                                 *
 *            Texas Instruments                                             * 
 *            Jan 5, 2012                                                 *
***************************************************************************/
#include <stdio.h>
#include <string.h>
#include "Keystone_common.h"
#include "Keystone_EMIF16_init.h"
#include "EMIF_NOR_FLASH_test.h"

#define FLASH_TEST_BUFFER_SIZE 	(1024*1024)
#pragma DATA_SECTION(testBuf,".far.testBuf") 
Uint8	testBuf[FLASH_TEST_BUFFER_SIZE];

EMIF16_CE_Config gNorCeCfg;	/*the configuration for the CE of NOR FLASH*/
EMIF16_CE_Config gNandCeCfg;/*the configuration for the CE of NAND FLASH*/
EMIF16_Config gEmif16Cfg;
void EMIF_init()
{
	memset(&gNorCeCfg, 0, sizeof(gNorCeCfg));
	memset(&gNandCeCfg, 0, sizeof(gNandCeCfg));
	memset(&gEmif16Cfg, 0, sizeof(gEmif16Cfg));
	
	/*configuraiton for the CE of NOR FLASH*/
	gNorCeCfg.busWidth= EMIF_BUS_16BIT,
	gNorCeCfg.opMode  = NOR_ASRAM_MODE,
	gNorCeCfg.strobeMode = SS_STROBE,  
	gNorCeCfg.waitMode = EMIF_WAIT_NONE,
	gNorCeCfg.wrSetup = CSL_EMIF16_A1CR_WSETUP_RESETVAL,
	gNorCeCfg.wrStrobe = CSL_EMIF16_A1CR_WSTROBE_RESETVAL,
	gNorCeCfg.wrHold= CSL_EMIF16_A1CR_WHOLD_RESETVAL,
	gNorCeCfg.rdSetup= CSL_EMIF16_A1CR_RSETUP_RESETVAL,
	gNorCeCfg.rdStrobe= CSL_EMIF16_A1CR_RSTROBE_RESETVAL,
	gNorCeCfg.rdHold= CSL_EMIF16_A1CR_RHOLD_RESETVAL,
	gNorCeCfg.turnAroundCycles= CSL_EMIF16_A1CR_TA_RESETVAL,
	gNorCeCfg.nor_pg_Cfg= NULL,        
	gEmif16Cfg.ceCfg[0] = &gNorCeCfg;

	/*configuraiton for the CE of NAND FLASH*/
	gNandCeCfg.busWidth= EMIF_BUS_16BIT,
	gNandCeCfg.opMode  = NAND_MODE,
	gNandCeCfg.strobeMode = SS_STROBE,  
	gNandCeCfg.waitMode = EMIF_WAIT0,
	gNandCeCfg.wrSetup = CSL_EMIF16_A1CR_WSETUP_RESETVAL,
	gNandCeCfg.wrStrobe = CSL_EMIF16_A1CR_WSTROBE_RESETVAL,
	gNandCeCfg.wrHold= CSL_EMIF16_A1CR_WHOLD_RESETVAL,
	gNandCeCfg.rdSetup= CSL_EMIF16_A1CR_RSETUP_RESETVAL,
	gNandCeCfg.rdStrobe= CSL_EMIF16_A1CR_RSTROBE_RESETVAL,
	gNandCeCfg.rdHold= CSL_EMIF16_A1CR_RHOLD_RESETVAL,
	gNandCeCfg.turnAroundCycles= CSL_EMIF16_A1CR_TA_RESETVAL,
	gNandCeCfg.nor_pg_Cfg= NULL,        
	gEmif16Cfg.ceCfg[1] = &gNandCeCfg;

	gEmif16Cfg.wait0Polarity= EMIF_LOW_WAIT;
	gEmif16Cfg.wait1Polarity= EMIF_LOW_WAIT;
	gEmif16Cfg.maxWait = CSL_EMIF16_AWCCR_MAXEXTWAIT_RESETVAL;
	
	KeyStone_EMIF16_init(&gEmif16Cfg);

}

void main( void )
{
	TDSP_Type dspType;

    // System init
    CACHE_setL1PSize(CACHE_L1_32KCACHE);
    CACHE_setL1DSize(CACHE_L1_32KCACHE);
    CACHE_setL2Size(CACHE_0KCACHE);
    CACHE_invAllL1p(CACHE_WAIT);
    CACHE_wbInvAllL1d(CACHE_WAIT);
    CACHE_wbInvAllL2(CACHE_WAIT);

	dspType= Get_dsp_type();
	if(TCI6608==dspType)
	{
		//DSP core speed
		//KeyStone_main_PLL_init (20, 1); 	//for 50MHz input clock
		KeyStone_main_PLL_init (10, 1); 	//for 100MHz input clock
	}
	else if(TCI6614==dspType||TCI6616==dspType||TCI6618==dspType)
	{
		//DSP core speed
		KeyStone_main_PLL_init (236, 29); 	//for 122.88MHz input clock
	}
	else
	{
		puts("Unknown DSP type!");
		return;
	}

	EMIF_init();

	NOR_FLASH_test((Uint32)testBuf, FLASH_TEST_BUFFER_SIZE);

	//NAND_FLASH_test();
}

