/* --------------------------------------------------------------------------
  FILE      : nand_table.h
  PURPOSE   : NAND Table header file
  PROJECT   : DaVinci User Boot-Loader and Flasher
  AUTHOR    : Daniel Allred
  DESC      : Header file to define the structure used in device nand tables.
                that list ROM-supported NAND devices
-------------------------------------------------------------------------- */

#ifndef _NAND_TABLE_H_
#define _NAND_TABLE_H_

#include <tistdtypes.h>

// Prevent C++ name mangling
#ifdef __cplusplus
extern far "c" {
#endif

/************************************************************
* Global Macro Declarations                                 *
************************************************************/


/***********************************************************
* Global Typedef declarations                              *
***********************************************************/

// NAND_DEVICE structure - holds supported devices' params
typedef struct _NAND_TABLE_DEVICE_OBJECT_
{
  Uint8   devID;              // DeviceID
  Uint16  numBlocks;          // number of blocks in device
  Uint8   pagesPerBlock;      // page count per block
  Uint16  bytesPerPage;       // byte count per page (include spare)
}
NAND_TABLE_DeviceObj, *NAND_TABLE_DeviceHandle;


/************************************************************
* Global Function Declarations                              *
************************************************************/


/***********************************************************
* End file                                                 *
***********************************************************/

#ifdef __cplusplus
}
#endif

#endif //_NAND_TABLE_H_

/* --------------------------------------------------------------------------
  HISTORY
    v1.00 - DJA - 12-Feb-2008
      Initial release
-------------------------------------------------------------------------- */

