/* --------------------------------------------------------------------------
  FILE        : util.h
  PURPOSE     : Misc. utility header file
  PROJECT     : DaVinci Flashing Utilities
  AUTHOR      : Daniel Allred
  DESC        : Header for UTIL module
 ----------------------------------------------------------------------------- */ 

#ifndef _UTIL_H_
#define _UTIL_H_

#include <tistdtypes.h>

// Prevent C++ name mangling
#ifdef __cplusplus
extern far "c" {
#endif

/***********************************************************
* Global Macro Declarations                                *
***********************************************************/
#define DEVICE_DDR_RAM_SIZE      (0x10000000u)

#define DEVICE_MAX_IMAGE_SIZE    (0x02000000u)

#define DEVICE_DDR_START_ADDR    (0x82000000u)

#define DEVICE_DDR_END_ADDR      ((RAM_START_ADDR + DDR_RAM_SIZE))

#define DEVICE_EMIF_FIRST_CE_START_ADDR   (0x70000000)
#define DEVICE_EMIF_INTER_CE_REGION_SIZE  (0x4000000)
#define DEVICE_EMIF_NUMBER_CE_REGION		(4)

#define FLASH_BUS_WIDTH_1_BYTE 		1
#define FLASH_BUS_WIDTH_2_BYTES 	2

#define ENDIAN_SWAP(a) (((a&0xFF)<<24)|((a&0xFF0000)>>8)|((a&0xFF00)<<8)|((a&0xFF000000)>>24))

#ifndef E_PASS
  #define E_PASS    (0x00000000u)
#endif
#ifndef E_FAIL
  #define E_FAIL    (0x00000001u)
#endif
#ifndef E_TIMEOUT
  #define E_TIMEOUT (0x00000002u)
#endif
/*
 * Null declaration 
 */
#ifndef NULL
  #define NULL    (0)
#endif

// Boolean declarations
#ifndef TRUE
  #define TRUE    ((Bool) 1)
#endif
#ifndef FALSE
  #define FALSE   ((Bool) 0)
#endif

// Generic bit mask defines
#ifndef BIT0
  #define BIT0    (0x00000001)
  #define BIT1    (0x00000002)
  #define BIT2    (0x00000004)
  #define BIT3    (0x00000008)
  #define BIT4    (0x00000010)
  #define BIT5    (0x00000020)
  #define BIT6    (0x00000040)
  #define BIT7    (0x00000080)
  #define BIT8    (0x00000100)
  #define BIT9    (0x00000200)
  #define BIT10   (0x00000400)
  #define BIT11   (0x00000800)
  #define BIT12   (0x00001000)
  #define BIT13   (0x00002000)
  #define BIT14   (0x00004000)
  #define BIT15   (0x00008000)
  #define BIT16   (0x00010000)
  #define BIT17   (0x00020000)
  #define BIT18   (0x00040000)
  #define BIT19   (0x00080000)
  #define BIT20   (0x00100000)
  #define BIT21   (0x00200000)
  #define BIT22   (0x00400000)
  #define BIT23   (0x00800000)
  #define BIT24   (0x01000000)
  #define BIT25   (0x02000000)
  #define BIT26   (0x04000000)
  #define BIT27   (0x08000000)
  #define BIT28   (0x10000000)
  #define BIT29   (0x20000000)
  #define BIT30   (0x40000000)
  #define BIT31   (0x80000000)
#endif
 
/***********************************************************
* Global Typedef declarations                              *
***********************************************************/


/***********************************************************
* Global Function Declarations                             *
***********************************************************/


/***********************************************************
* End file                                                 *
***********************************************************/

#ifdef __cplusplus
}
#endif

#endif //_UTIL_H_

/* --------------------------------------------------------------------------
  HISTORY
	  v1.00  -  DJA  -  07-Nov-2007
      Initial release 
 ----------------------------------------------------------------------------- */

